import 'package:equatable/equatable.dart';
import 'package:flutter/material.dart';
import 'package:hydrated_bloc/hydrated_bloc.dart';

import 'package:spotifyplaylistgenerator/models/activity/activity.dart';
import 'package:spotifyplaylistgenerator/models/settings/settings_global.dart';

part 'activity_state.dart';

class ActivityCubit extends HydratedCubit<ActivityState> {
  ActivityCubit()
      : super(ActivityState(
          currentActivity: Activity.createNull(),
        ));

  void updateState(Activity activity) {
    emit(ActivityState(
      currentActivity: activity,
    ));
  }

  void refresh() {
    final Activity activity = Activity(
      // Settings
      globalSettings: state.currentActivity.globalSettings,
      // State
      isConnected: state.currentActivity.isConnected,
      // Base data
      userId: state.currentActivity.userId,
      // Activity data
      token: state.currentActivity.token,
    );
    // activity.dump();

    updateState(activity);
  }

  void startNewActivity({
    required GlobalSettings globalSettings,
  }) {
    final Activity newActivity = Activity.createNew(
      // Settings
      globalSettings: globalSettings,
    );

    newActivity.dump();

    updateState(newActivity);
    refresh();
  }

  void quitActivity() {
    state.currentActivity.isConnected = false;
    refresh();
  }

  void resumeSavedActivity() {
    state.currentActivity.isConnected = true;
    refresh();
  }

  void deleteSavedActivity() {
    state.currentActivity.isConnected = false;
    refresh();
  }

  @override
  ActivityState? fromJson(Map<String, dynamic> json) {
    final Activity currentActivity = json['currentActivity'] as Activity;

    return ActivityState(
      currentActivity: currentActivity,
    );
  }

  @override
  Map<String, dynamic>? toJson(ActivityState state) {
    return <String, dynamic>{
      'currentActivity': state.currentActivity.toJson(),
    };
  }
}
